// lessovw.cpp : implementation of the Clesson3View class
//

#include "stdafx.h"
#include "lesson3.h"

#include "lessodoc.h"
#include "lessovw.h"

#include "amber.hpp"
#include "spin.hpp"
#include "spingrav.hpp"
#include <math.h>

#define X 0
#define Y 1
#define Z 2

// Make the cubes global so that they can be deleted
//
spinClass *cubes[5];
spinGravClass *cubesG[5];

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

void randomColorFunc(void *obj)
{
	geometryClass *g = (geometryClass *)obj;
	F4 col;
	col[0] = (float)(rand() % 10)/10.0;
	col[1] = (float)(rand() % 10)/10.0;
	col[2] = (float)(rand() % 10)/10.0;
	col[3] = 1.0;

	g->setAllColor(col);
}

void myInit()
{
	V3 pos;
	int i;

	// Create all spinning cubes with random
	// spin axis.
	//
	for (i=0; i<5; i++) {
		cubes[i] = new spinClass( 0.17, 
										  (rand() % 3),
										  1,1,1);
		// Set them in random positions
		//
		pos[X] = (Vres)(rand() % 10) - 5.0;
		pos[Y] = (Vres)(rand() % 10) - 5.0;
		pos[Z] = (Vres)(rand() % 10) - 5.0;
		cubes[i]->setPosition(pos);

	}

	// Add an "action" behavior to the
	// first cube.
	//
	cubes[0]->addAction(randomColorFunc, 10); 
}

void myInit2()
{
	V3 pos;
	int i;
	V3 vel;

	// Create all spinning gravity cubes. 
	//
	for (i=0; i<5; i++) {
		cubesG[i] = new spinGravClass();
		pos[X] = (Vres)(rand() % 10) - 5.0;
		pos[Y] = (Vres)(rand() % 10) - 5.0;
		pos[Z] = (Vres)(rand() % 10) - 5.0;
		cubesG[i]->setPosition(pos);

		// Set kinematic pos.
		cubesG[i]->setPos(pos);         

		// Add initial random velocity
		//
		vel[X] = ((Vres)(rand() % 10) - 5.0)/5.0;
		vel[Y] = ((Vres)(rand() % 10) - 5.0)/5.0;
		vel[Z] = ((Vres)(rand() % 10) - 5.0)/5.0;
		cubesG[i]->setVel(vel);
	}
}

void pointGravity(lawClass *obj) {

	V3 accel;

	// Get the position of the object
	//
	V3_copy(accel, obj->kinematicClass::position);

	// Create acceleration vector
	V3_multk(accel, -0.25);

	obj->setAccel(accel);
}
/////////////////////////////////////////////////////////////////////////////
// Clesson3View

IMPLEMENT_DYNCREATE(Clesson3View, CView)

BEGIN_MESSAGE_MAP(Clesson3View, CView)
	//{{AFX_MSG_MAP(Clesson3View)
	ON_WM_SIZE()
	ON_WM_KEYDOWN()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Clesson3View construction/destruction

Clesson3View::Clesson3View()
{
	// TODO: add construction code here

	// Initialize Amber channel
	ch = NULL;

	// Initialize Amber mouse
	mouse = NULL;

}

Clesson3View::~Clesson3View()
{
	int i;

	// Delete mouse
	delete mouse;

	for (i=0; i<5; i++) {
		delete cubes[i];
		delete cubesG[i];
	}
}

/////////////////////////////////////////////////////////////////////////////
// Clesson3View drawing

void Clesson3View::OnDraw(CDC* pDC)
{
	Clesson3Doc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// Clesson3View printing

BOOL Clesson3View::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void Clesson3View::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void Clesson3View::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// Clesson3View diagnostics

#ifdef _DEBUG
void Clesson3View::AssertValid() const
{
	CView::AssertValid();
}

void Clesson3View::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

Clesson3Doc* Clesson3View::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(Clesson3Doc)));
	return (Clesson3Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// Clesson3View message handlers

void Clesson3View::OnInitialUpdate() 
{
	// TODO: Add your specialized code here and/or call the base class

	// Retrieve the HWND pointer to this window
	//
	HWND hwnd = GetSafeHwnd();
	V3 chPos;

	// Create the channel if it has not already been created
	//
	if (ch == NULL) {
		ch = new channelClass((void *)hwnd);
		myInit();
		myInit2();

		universeClass *univ = amber->getCurrentUniverse();
		univ->addLaw(pointGravity);
	}

	// Create the mouse if it has not already been created
	// Also, attach the channel to the mouse.
	if (mouse == NULL) {
		mouse = new mouseClass(m_hWnd);
		mouse->attachChannel(ch);
		V3_set(chPos, 0.0, 0.0, 20.0);
		ch->setPosition(chPos);
	}

	CView::OnInitialUpdate();
}

void Clesson3View::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	
	// Adjust Amber channel when window size is changed.  Return if
	// channel has not been created or this window is not visible.
	//
	if (!IsWindowVisible() || ch==NULL) return;
	ch->resetPerspective();
}

BOOL Clesson3View::PreCreateWindow(CREATESTRUCT& cs) 
{
	// TODO: Add your specialized code here and/or call the base class
	
	// Set the style of the window to clip children and siblings.  This
	// is a requirement of OpenGL windows.
	//	
	cs.style = cs.style | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	return CView::PreCreateWindow(cs);
}

int useDisplayLists=0;
void Clesson3View::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	int i;
		
	CView::OnKeyDown(nChar, nRepCnt, nFlags);

	switch (nChar) {
		case 'D':
			useDisplayLists = !useDisplayLists;
			if (useDisplayLists) {
				for (i=0; i<5; i++) {
					cubes[i]->enableState(RENDER_DISPLAYLIST);
					cubesG[i]->enableState(RENDER_DISPLAYLIST);
				}
			}
			else {
				for (i=0; i<5; i++) {
					cubes[i]->disableState(RENDER_DISPLAYLIST);
					cubesG[i]->disableState(RENDER_DISPLAYLIST);
				}
			}
			break;
	}
}
